#!/usr/bin/perl
use strict;
use warnings;
use FileHandle;
use Getopt::Std;
use Cwd qw(getcwd);

my %opts;
getopts('hq:a:b:c:g:o:', \%opts);

##################################
#	Author: Sinje Neukirchen
#
#
#	files needed:
#	- table with query accession and protein name 
#
#		example structure, with spaces or tabs as delimiter
#		protein_name	genome_accession	protein_accession
#
#	- blastp results with output format '7 std qlen slen qcovs'
#		a) genome vs itself 	(filename: genome.blastp)
#		b) queries vs genome	(filename: queries_vs_genome.blastp)
#		c) genome vs queries	(filename: genome_vs_queries.blastp")
#
##################################
my ($in, $fh, $out, $fho, $line, $home, $dom, $c, $i2, $s2, $q2, $e2, $qc2);
my ($identity, $evalue, $qcovs, $genome, $query, $subject, $prot,  $copy);
my ($global, $alen, $qlen, $slen, $max, $name, $folder);
my (@list, @todo, @hit);
my (%genomes, %query2genome, %id2prot, %pairs2, %pairs, %qhit, %hits, %done, %within, %todo);


if($opts{h}){
	usage();
	exit(0);
}
if(!$opts{q} || !$opts{a}  || !$opts{b} || !$opts{c}  || !$opts{g} ){
	warn "\nERROR:\tInput missing\n\n\n";
	usage();
	exit(1);
}

if(!$opts{o}){
	$home = getcwd;
}else{
	$home = $opts{o};
}

$in = $opts{q};
$fh = FileHandle->new;
if ($fh->open("< $in")) {
	while($line=<$fh>){
		chomp($line);
		@list=split(/\s+/, $line);
		$prot = $list[0];
		$genome = $list[1];
		$query=$list[2];
		$id2prot{$query}=$prot;
		$query2genome{$query}=$genome;
	}
	$fh->close;
}else{
	die "No such file $in\n";
}



$in = $opts{a};	#genome.blastp
$fh = FileHandle->new;
if($fh->open("< $in")){
	while($line=<$fh>){
		chomp($line);
		if(substr($line,0,1) eq '#'){
			next;
		}
		@list=split(/\s+/, $line);
		$subject=$list[1];
		$query=$list[0];
		$identity=$list[2];
		$alen=$list[3];
		$evalue=$list[10];
		$qlen=$list[12];
		$slen=$list[13];
		$qcovs=$list[14];
		if(! $slen){
			die "$in\n$line\n";
		}
		if($qlen>$slen){
			$max = $qlen;
		}else{
			$max=$slen;
		}
		$global = $identity/($max/$alen);
		$global = sprintf("%7.3f", $global);
		if($global >= 70 && $evalue <= 1e-10){
			if($query ne $subject){
				$within{$query}{$subject}=$identity."\t".$evalue."\t".$qcovs."\t".$global;
				$within{$subject}{$query}=$identity."\t".$evalue."\t".$qcovs."\t".$global;
			}
		}
	}
	$fh->close;
}else{
	die "No such file $in\n";
}

$genome = $opts{g};
	

#queries vs genome, 1. direction
$in = $opts{b};
if($fh->open("< $in")){
	while($line=<$fh>){
		chomp($line);
		if(substr($line,0,1) eq '#'){
			next;
		}
		@list=split(/\s+/, $line);
		$query=$list[0];
		$subject=$list[1];
		$identity=$list[2];
		$alen=$list[3];
		$evalue=$list[10];
		$qlen=$list[12];
		$slen=$list[13];
		$qcovs=$list[14];
		if(! $slen){
			die "$in\n$line\n";
		}
		if($qlen>$slen){
			$max = $qlen;
		}else{
			$max=$slen;
		}
		$global = $identity/($max/$alen);
		$global = sprintf("%7.3f", $global);
		if($global >= 25 && $evalue <= 1e-10 ){
			$pairs2{$query}{$subject}=$identity."\t".$evalue."\t".$qcovs."\t".$global;
			$query2genome{$subject}=$genome;
		}
		if($global >= 25 && $evalue <= 1e-10 && !exists $qhit{$query}){
			$qhit{$query}=$subject."\t".$identity."\t".$evalue."\t".$qcovs."\t".$global;
			$pairs{$query}{$subject}=$identity."\t".$evalue."\t".$qcovs."\t".$global;
			$query2genome{$subject}=$genome;
		}elsif($global >= 25 && $evalue <= 1e-10 && exists $qhit{$query}){
			@hit= split(/\t/, $qhit{$query});
			$i2=$hit[4];	#or global
			$s2=$hit[0];
			$e2 = $hit[2];
			$qc2 = $hit[3];
			if($global>$i2 && $evalue<$e2 && $qcovs>$qc2){
				delete($pairs{$query}{$s2});
				$pairs{$query}{$subject}=$identity."\t".$evalue."\t".$qcovs."\t".$global;
				$query2genome{$subject}=$genome;
				$qhit{$query}=$subject."\t".$identity."\t".$evalue."\t".$qcovs."\t".$global;
			}
		}
	}	
	$fh->close;
}else{
	die "No such file $in\n";
}


if(scalar (keys %pairs) == 0){
	die "No hits found $opts{g}\n";
}


#genome vs queries -> 2. direction
$in = $opts{c};
$fh = FileHandle->new;
if($fh->open("< $in")){
	while($line=<$fh>){
		chomp($line);
		if(substr($line,0,1) eq '#'){
			next;
		}
		@list=split(/\s+/, $line);
		$subject=$list[0];
		$query=$list[1];
		$identity=$list[2];
		$alen=$list[3];
		$evalue=$list[10];
		$qlen=$list[12];
		$slen=$list[13];
		$qcovs=$list[14];
		if(! $slen){
			die "$in\n$line\n";
		}
		if($qlen>$slen){
			$max = $qlen;
		}else{
			$max=$slen;
		}
		$global = $identity/($max/$alen);
		$global = sprintf("%7.3f", $global);
		if(exists $pairs{$query}{$subject}){
			if($global >= 25 && $evalue <= 1e-10 && !exists $done{$subject}){
				$done{$subject}=$query."\t".$identity."\t".$evalue."\t".$qcovs."\t".$global;
				$hits{$query}{$subject}++;
			}elsif($global >= 25 && $evalue <= 1e-10 && exists $done{$subject}){
				@hit= split(/\t/, $done{$subject});
				$i2=$hit[4];	#or global
				$q2=$hit[0];
				$e2 = $hit[2];
				$qc2 = $hit[3];
				if($global>$i2 && $evalue<$e2 && $qcovs>$qc2){
					delete($hits{$q2}{$subject});
					$done{$subject}=$query."\t".$identity."\t".$evalue."\t".$qcovs."\t".$global;
					$hits{$query}{$subject}++;
				}
			}
		}
	}	
	$fh->close;
}
else{
	die "No such file $in\n";
}
$c =0;

$out = "$home/$genome.rBBH.txt";
open($fho, "> $out") || die "No $out\n";
print $fho "#prot\tquery\tquery_genome\tsubject\tgenome\tlocal_ident\te-value\tqcovs\tglobal_ident\n";
foreach $query (sort {$id2prot{$a} cmp $id2prot{$b}} keys %hits){
	foreach $subject (sort keys %{$hits{$query}}){
		print $fho "$id2prot{$query}\t$query\t$subject\t$query2genome{$subject}\t$pairs{$query}{$subject}\n";
		$c++;
		if(exists $within{$subject}){
			foreach $copy (keys %{$within{$subject}}){
				if(exists $pairs2{$query}{$copy} && !exists $done{$copy}){
					$done{$copy}=$query."\t".$identity."\t".$evalue."\t".$qcovs."\t".$global;
					print $fho "$id2prot{$query}\t$query\t$copy\t$query2genome{$copy}\t$pairs2{$query}{$copy}\tCOPY\n";
					$c++;
				}
			}
		}
	}
	
}	
close($fho);


if($c>0){
	print "$out\n";
}else{
	system("rm $out");
	print "No hits found $opts{g}\n";

}

### usage ###
sub usage {
print <<END_OF_USAGE;

DESCRIPTION:
	perl filter_blastp_rBBH.pl  
	reciprocal best blast hit filtering

USAGE:
  perl filter_blastp_rBBH.pl -g <genome_acc> -a <genome.blastp> -b <queries_vs_genome.blastp> -c <genome_vs_queries.blastp> -q <queries_list.txt>
  \[-o <path_to_output_directory>\]
  \[-h\]

END_OF_USAGE
}


