#!/usr/bin/perl

use strict;
use warnings;
use FileHandle;
use Getopt::Std;
use Cwd qw(getcwd);
my %opts;
getopts('hg:b:n:o:', \%opts);

##################################
#	Author: Sinje Neukirchen
#
#
#	files needed:
#	- reciprocal best blast hit table
#		example structure: prot   query   query_genome    subject genome  local_ident     e-value qcovs   global_ident
#	- genome protein sequences
#	
#
##################################
my ($in, $fh, $fho, $out, $line, $in2, $fh2, $line2, $i, $al, $m, $d, $home);
my ($id, $prot, $tmp, $c, $genome, $name);
my (@list, @list2, @todo, @prot);
my (%todo, %id2prot, %head2seq);



if($opts{h}){
	usage();
	exit(0);
}
if(!$opts{b} || !$opts{n}  || !$opts{g}){
	warn "\nERROR:\tInput missing\n\n\n";
	usage();
	exit(1);
}

if(!$opts{o}){
	$home = getcwd;
}else{
	$home = $opts{o};
}

$name = $opts{n};

$name =~s/\W+/_/g;
$name =~s/__/_/g;


$in = $opts{b};
$fh = FileHandle->new;
if($fh->open("< $in")){
	while($line=<$fh>){
		chomp($line);
		@list=split(/\s+/, $line);
		if($line=~m/^#/){
			next;
		}
		$prot = $list[0];
		$id = $list[3];
		$genome=$list[4];
		$id2prot{$id}=$prot;
	}
	
	$fh->close;
}else{
	die "No such file $in\n";
}

$in = $opts{g};
$fh = FileHandle->new;
if($fh->open("< $in")){
	while($line=<$fh>){
		chomp($line);
		if($line =~m/>(\S+)/){
			$id = $1;
		}else{
			$head2seq{$id}.=$line;
		}
	}
	$fh->close;
}else{
	die "No such file $in\n";
}

$out = "$home/$genome.rBBH.faa";
open($fho, "> $out") || die "No $out\n"; 
foreach $id (sort keys %id2prot){
	print $fho ">$id"."__$name\n$head2seq{$id}\n";
}
close($fho);
print "$out\n";

### usage ###
sub usage {
print <<END_OF_USAGE;

DESCRIPTION:
	perl get_seq.pl  
	extracts sequences from a genome
USAGE:
  perl get_seq.pl -g <genome.faa> -b <rBBH.txt> -n <species_name>  
  \[-o <outfile> \]
  \[-h\]

END_OF_USAGE
}


