#!/usr/bin/perl

use strict;
use warnings;
use FileHandle;
use Getopt::Std;
use Cwd qw(getcwd);
my %opts;
getopts('hf:b:o:', \%opts);

##################################
#	Author: Sinje Neukirchen
#
#
#	files needed:
#	- reciprocal best blast hit table
#		example structure: prot   query   query_genome    subject genome  local_ident     e-value qcovs   global_ident
#	- genome featue table or IMG gff 
#	
#
##################################

my $d = 2;

my ($in, $fh, $fho, $file, $out, $fho2, $out2, $line, $in2, $fh2, $line2, $max, $i, $al, $len, $aalen, $count, $v, $o, $p, $s, $e, $col, $color, $home);
my ($id, $prot, $tmp, $c, $genome, $phenotype, $v2, $id2, $db);
my ($chrom, $chrom2, $start, $end, $strand, $anno, $m, $n, $dsrab, $king, $name);
my (@list, @list2, @todo, @sort_prot, @h, @p, @an, $el);
my (%redundant, %addD, %FP, %genome2id, %todo, %id2prot, %id2clust2, %sort, %db, %count, %v2id, %v2o,  %do, %id2clust, %tmp, %hits, %tax, %prot_col, %org, %done);


if($opts{h}){
	usage();
	exit(0);
}
if(!$opts{b} ||  !$opts{f}){
	warn "\nERROR:\tInput missing\n\n\n";
	usage();
	exit(1);
}



$in = $opts{b};
open($fh, "< $in") or die "$! $in\n";
while($line=<$fh>){
	chomp($line);
	if($line=~m/^#/){
		next;
	}
	@list=split(/\s+/, $line);
	$id=$list[3];
	$prot=$list[0];
	$genome = $list[4];
	$id2clust{$id}=$prot;
}
close($fh);



$in = $opts{f};
$v=0;
$fh = FileHandle->new;
open($fh, "< $in") or die "$! $in\n";
while($line=<$fh>){
	chomp($line);
	if($line =~m/^CDS/){
		@list=split(/\t/, $line);
		$chrom = $list[6];
		$start = $list[7];
		$end = $list[8];
		$strand = $list[9];
		$id = $list[10];
		$anno = $list[13];
		$len = $list[17];
		$aalen = $len/3;
		$end = $start+$len-1;
			
		if($line =~ m/pseudo/ && $id !~m/\w/){
			$id = "pseudo_$v";
			$aalen = int($len/3);
			}
		if(exists $id2clust{$id}){
			push(@sort_prot, $v);
			$hits{$v}="$chrom\t$start\t$end\t$aalen\t$strand\t$id\t$anno\t$id2clust{$id}";
			}else{
			$hits{$v}="$chrom\t$start\t$end\t$aalen\t$strand\t$id\t$anno\t---";
		}
		$v2id{$v}=$id;
		$v++;
	}elsif($line =~m/^gene\s+other/){
		@list=split(/\t/, $line);
		$chrom = $list[6];
		$start = $list[7];
		$end = $list[8];
		$strand = $list[9];
		$id = "pseudo_$v";
		$anno = $list[14];
		$len = $list[17];
		$aalen = int($len/3);
		$end = $start+$len-1;
		if(exists $id2clust{$id}){
			push(@sort_prot, $v);
			$hits{$v}="$chrom\t$start\t$end\t$aalen\t$strand\t$id\t$anno\t$id2clust{$id}";
		}else{
			$hits{$v}="$chrom\t$start\t$end\t$aalen\t$strand\t$id\t$anno\t---";
		}
		$v2id{$v}=$id;
		$v++;
	}elsif($line =~m/\s+CDS\s+\d+.+ID=/){
		@list=split(/\t/, $line);
		$chrom = $list[0];
		$start = $list[3];
		$end = $list[4];
		$strand = $list[6];
		$tmp =$list[8];
		if($tmp =~m/ID=(.+);locus_tag=(.+);product=(.+)$/){
			$id = $1;
			$anno = $3;
			$anno=~s/\s+/_/g;
		}
		$len = $end-$start+1;
		$aalen = int($len/3);
		
		if($line =~ m/pseudo/ && $id!~m/\w/){
			$id = "pseudo_$v";
			$aalen = int($len/3);
		}
		if(exists $id2clust{$id}){
			push(@sort_prot, $v);
			$hits{$v}="$chrom\t$start\t$end\t$aalen\t$strand\t$id\t$anno\t$id2clust{$id}";
		}else{
			$hits{$v}="$chrom\t$start\t$end\t$aalen\t$strand\t$id\t$anno\t---";
		}
		$v2id{$v}=$id;
		$v++;
	}
}
close($fh);

@p =  (sort {$b <=> $a} keys %hits);
$max = $p[0];
if(scalar(@sort_prot) < 2){
	@sort_prot=();
	undef(%hits);
	undef(%tmp);
	undef(%org);
	undef(%count);
	undef(%do);
	undef(%v2id);
	undef(%v2o);
	die "$genome\tless than 2 hits\n";
}

foreach $v (@sort_prot){
	for($i=0; $i<=$d; $i++){
		if($v-$i>0){
			@h=split(/\t/, $hits{$v});
			$chrom = $h[0]; 
			@h=split(/\t/, $hits{$v-$i});
			$chrom2 = $h[0]; 
			if($chrom eq $chrom2){
				$tmp{$v-$i}++;
			}
			if(exists $hits{$v+$i} && ($v+$i)<=$max){
				@h=split(/\t/, $hits{$v});
				$chrom = $h[0]; 
				@h=split(/\t/, $hits{$v+$i});
				$chrom2 = $h[0]; 
				if($chrom eq $chrom2){
					$tmp{$v+$i}++;
				}
			}
		}elsif($v-$i==0){
			@h=split(/\t/, $hits{$v});
			$chrom = $h[0]; 
			$tmp{$v-$i}++;
		}
	}	
}
	
@p =  (sort {$a <=> $b} keys %tmp);
$c=0;
$o=0;
for($i=0; $i<scalar(@p); $i++){
	$v=$p[$i];
	if($v-$c>1 && $c != 0){	
		$o++;
	}
	if($i>0){
		@h=split(/\t/, $hits{$c});
		$chrom = $h[0]; 
		@h=split(/\t/, $hits{$v});
		$chrom2 = $h[0]; 
		if($chrom ne $chrom2){
			$o++;
		}
	}	
	$count{$o}{$v}++;
	$v2o{$v}=$o;

	$c=$v;
}

foreach $o (sort {$a <=>$b} keys %count){
	$c=0;
	if(scalar(keys %{$count{$o}}) == 5){
		foreach $v ( keys  %{$count{$o}}){
			if(exists $id2clust{$v2id{$v}}){
				$c=1;
			}
		}
	}else{
		$c=scalar(keys %{$count{$o}});
		foreach $v ( keys  %{$count{$o}}){
			if(!exists $id2clust{$v2id{$v}}){
				$c=$c-1;
			}
		}
	}
	if($c >0){
		foreach $v ( keys  %{$count{$o}}){
			$do{$v}++;
		}
	}
}


$o=1;
$c=0;

if(!$opts{o}){
	$home = getcwd;
	$out2 = "$home/$genome.R.txt";
}else{
	$out2 = $opts{o};
}

$fho2 = FileHandle->new;
if($fho2->open("> $out2")){
	print $fho2 "name\tstart\tend\tstrand\tfill\n";
	$start=0;
	$end = 0;
	$chrom = "";
	for($i=0; $i<scalar(@p); $i++){
		$v=$p[$i];
		if(!exists $hits{$v}){
			die "$genome\t$v\n"
		}
		@h=split(/\t/, $hits{$v});
		$o=$v2o{$v};
		@list = split(/\t/, $hits{$v});
		$id = $list[5];
		$prot = $list[7];
		$color = "wheat";
		if($prot eq "---"){
			$anno = $list[6];
			$anno=~s/[\W ]+/_/g;
			$prot = substr($anno,0,15);
		}
		$strand = $list[4];
		$s=$list[1];
		$e=$list[2];
		$id = $list[5];
		if($i == 0){	
			$start = 0;
			$end = int(($e-$s)/3);
		}else{
			if($v-$p[$i-1]<=$d){
				$start = $end + 1;
				$end = $start + int(($e-$s)/3);
			}else{
				$start = $end + 1000 + ($v-$p[$i-1]);
				$end = $start + int(($e-$s)/3);
			} 					
		}
		if($v-$c>1 && $c != 0){
			$o=$v2o{$v};
		}
		print $fho2 "$prot\t$start\t$end\t$strand\t$color\n";
		$c=$v;
	}
	$fho2->close;
}else{
	die "No such file $out\n";
}


print "$out2\n";




### usage ###
sub usage {
print <<END_OF_USAGE;

DESCRIPTION:
	perl synteny_table_for_R.pl  
	exctracts syntenic blast hits and produces input for R plotting

USAGE:
  perl synteny_table_for_R.pl  -g <genome_feature_table> -b <rBBH.txt>   
  \[-o <outfile> \]
  \[-h\]

END_OF_USAGE
}


