#!/usr/bin/perl
use strict;
use warnings;
use FileHandle;
use Getopt::Std;
use Cwd qw(getcwd);

my %opts;
getopts('hi:o:', \%opts);

##################################
#	Author: Sinje Neukirchen
#
#
#	files needed:
#	- tab-separated table (like TableS4)
#	genome_accession	taxID	Domain	 Phylum	Class	Order	Familiy	Species	Download_date	ftp_path
#
#	wget needs to be in your $PATH
#
##################################



my ($home, $in, $fh, $fho, $out, $line, $in2);
my ($id, $prot, $acc, $genome, $asm, $path, $type);
my (@list, @types);
my (%todo);

@types = (".faa.gz", "_feature_table.txt.gz");


if($opts{h}){
	usage();
	exit(0);
}
if(!$opts{i}){
	warn "\nERROR:\tList with genome accessions and ftp paths missing!\n\n\n";
	usage();
	exit(1);
}
if(!$opts{o}){
	$home = getcwd;
}else{
	$home = $opts{o};
}

##################################
#
#	list with genome accessions
#

$in = $opts{i};
$fh = FileHandle->new;
if($fh->open("< $in")){
	while($line=<$fh>){
		chomp($line);
		@list=split(/\t/, $line);
		if($line=~m/^#/){
			next;
		}
		$genome = $list[0];
		$path = $list[10];

		if($genome=~m/^GC/){
			$todo{$genome}=$path;
		}else{
			warn "$genome\tdownload it at $path with your account\n";
		}

	}
	
	$fh->close;
}else{
	die "No such file $in\n";
}


foreach $genome (keys %todo){
	$path = $todo{$genome};
	

	print "$genome\t$path\n";
	
	foreach $type (@types){
		$out = "$home/$genome$type";
		if($type =~m/faa/){
			$type = "_protein.faa.gz";
			$out = "$home/$genome.faa.gz";
		}
		print "wget $path/$genome*$type -O $out\n";
		system("wget \"$path/$genome\*$type\" -O $out");	
	}

}
### usage ###
sub usage {
print <<END_OF_USAGE;

DESCRIPTION:
	get_genomes.pl  - Automatic download of genomes from NCBI.
	These files will be downloaded: ".faa.gz", "_feature_table.txt.gz"

USAGE:
  perl get_genomes.pl -i <taxonomy_ftp_path>
  \[-o <path_to_output_directory>\]
  \[-h\]

 example:
  perl get_genomes.pl -i table_S4.txt 
END_OF_USAGE
}


